import inspect,copy
from ladon.exceptions.types import *
from ladon.types.ladontype import LadonType
from ladon.types.attachment import attachment
from ladon.compat import PORTABLE_STRING_TYPES

def get_userdef_class_attributes(cls,exclude_methods=True):
	base_attrs = dir(type('dummy', (object,), {}))
	this_cls_attrs = dir(cls)
	res = []
	for attr in this_cls_attrs:
		# attr == '__qualname__'
		# Python 3.3 __qualname__ (PEP 3155 -- Qualified name for classes and functions) fix
		if base_attrs.count(attr) or (exclude_methods and inspect.ismethod(getattr(cls,attr))) or attr == '__qualname__':
			continue
		res += [attr]
	return res


class TypeManager(object):

	"""
	This class is used by LadonServiceInfo to collect all types that a given
	service depends on. It also keeps track of the parse order so it is possible
	to create service type descriptors in the right order. This will prevent that
	an array of some type is not defined before the type itself.
	
		type_dict: holds all the parsed class-types that are derived from LadonType
		type_order: holds all parsed types in the correct parse order
		primitive_list: A list of all the primitive types a service depends on
	
	Primitive types are defined as all classes that don't inherit LadonType
	"""
	
	global_type_dict = {}
	
	def __init__(self):
		self.type_dict = {}
		self.type_order = []
		self.primitive_list = []
		self.has_lists = False
		self.has_dicts = False
		self.type_parse_order = 0

	
	def add_primitive_type(self,prim):
		if not self.primitive_list.count(prim):
			self.primitive_list += [prim]
	
	def analyze_class(self,cls):
		try:
			inspect.getmro(cls).count(LadonType)
		except Exception as e:
			raise NeedToDefineParseTimeException("class not LadonType inherited - %s" % str(e))
		type_key = cls
		if not type_key in self.type_dict:
			self.type_dict[type_key] = {
				'module': cls.__module__,
				'name': cls.__name__,
				'attributes': [] }
		else:
			return self.type_dict[type_key]
		type_container = self.type_dict[type_key]
		class_attrs = get_userdef_class_attributes(cls)
		for attr in class_attrs:
			attr_val = getattr(cls,attr)
			array_attr = False
			attr_props = {}
			if type(attr_val)==list and len(attr_val)==1:
				attr_val = attr_val[0]
				array_attr = True
				self.has_lists = True
			elif type(attr_val)==dict:
				# Dictionary type definitions for LadonTypes
				# New feature in Ladon 0.6.6
				try:
					temp_attr_val = attr_val['type']
				except AttributeError:
					raise NeedToDefineParseTimeException('Dictionary type definitions must contain the "type" key.\nclass: %s\nattr: %s' % (cls.__name__,attr))
				ok_default_types = []
				if 'nullable' in attr_val:
					if type(attr_val['nullable'])!=bool:
						raise NeedToDefineParseTimeException('Dictionary type definition nullable must be a boolean value.\nclass: %s\nattr: %s' % (cls.__name__,attr))
					attr_props['nullable'] = attr_val['nullable']
					if attr_props['nullable']:
						ok_default_types += [type(None)]
						
				if 'doc' in attr_val:
					temp_doc = attr_val['doc']
					if type(temp_doc) in PORTABLE_STRING_TYPES:
						attr_props['doc'] = [temp_doc]
					elif type(temp_doc)==list:
						attr_props['doc'] = []
						for doc_line in temp_doc:
							if type(doc_line) in PORTABLE_STRING_TYPES:
								attr_props['doc'] += [doc_line]
								
				if 'default' in attr_val:
					ok_default_types += [temp_attr_val]
					if type(attr_val['default']) not in ok_default_types:
						raise NeedToDefineParseTimeException('Default value does not match the attribute type.\nclass: %s\nattr: %s' % (cls.__name__,attr))
					attr_props['default'] = attr_val['default']
					has_default = True
					
				if 'filters' in attr_val:
					filters = attr_val['filters']
					for filter_type in ['incoming_raw','incoming','outgoing','outgoing_raw']:
						if filter_type in filters and hasattr(filters[filter_type],'__iter__') and len(filters[filter_type]):
							# Add attribute filter functions
							if 'filters' not in attr_props:
								attr_props['filters'] = {}
							attr_props['filters'][filter_type] = list(attr_val['filters'][filter_type])
							
				attr_val = temp_attr_val
			elif type(attr_val)!=type:
				raise NeedToDefineParseTimeException("class attributes on LadonTypes must be defined as types, lists.\nclass: %s\nattr: %s" % (cls.__name__,attr))
			if inspect.getmro(attr_val).count(LadonType):
				self.analyze_class(attr_val)
			else:
				self.add_primitive_type(attr_val)
				if attr_val==attachment:
					self.temp_require_multipart = True

			if array_attr:
				if not self.type_order.count([attr_val]):
					self.type_order += [[attr_val]]
				attr_val = [attr_val]
			attr_props['type'] = attr_val
			type_container['attributes'] += [(attr,attr_val,attr_props)]
		if not type_key in self.global_type_dict:
			self.global_type_dict[type_key] = copy.deepcopy(type_container)
		type_container['parse_order'] = self.type_parse_order
		self.type_order += [type_container]
		self.type_parse_order += 1
		return type_container

	def analyze_param(self,param):
		self.temp_require_multipart = False
		if [list,tuple].count(type(param)):
			# Parameter seems to be an array
			if len(param)!=1:
				raise NeedToDefineParseTimeException("Only one element is allowed when defining ladon arrays")
			if type(param[0])!=type:
				raise NeedToDefineParseTimeException("Only types are allowed as list-element when defining an array")
			
			if inspect.getmro(param[0]).count(LadonType):
				self.analyze_class(param[0])
			else:
				self.add_primitive_type(param[0])
				if param[0]==attachment:
					self.temp_require_multipart = True

			self.has_lists = True
			if not self.type_order.count([param[0]]):
				self.type_order += [[param[0]]]
		elif type(param)==type:
			if inspect.getmro(param).count(LadonType):
				self.analyze_class(param)
			else:
				self.add_primitive_type(param)
				if param==attachment:
					self.temp_require_multipart = True
		else:
			raise NeedToDefineParseTimeException("Ladonized param definition must be a LadonType or iterable")
		return self.temp_require_multipart



from ladon.exceptions.base import LadonException

class LadonDispatchException(LadonException):
	def __init__(self,text,ifname=None,sname=None):
		LadonException.__init__(self)
		self.text = text
		self.ifname = ifname
		self.sname = sname
		
	def __str__(self):
		if not self.ifname and not self.sname:
			return self.text
		elif not self.sname:
			return "\ninterface name: %s\n%s" % (self.ifname,self.text)
		else:
			return "\ninterface name: %s\nservice name: %s\n%s" % (self.ifname,self.sname,self.text)
			
class UndefinedInterfaceName(LadonDispatchException):
	def __init__(self,ifname,text):
		LadonDispatchException.__init__(self,text,ifname=ifname)


class UndefinedService(LadonDispatchException):
	def __init__(self,ifname,text):
		LadonDispatchException.__init__(self,text,ifname=ifname)


class UndefinedServiceMethod(LadonDispatchException):
	def __init__(self,ifname,sname,text):
		LadonDispatchException.__init__(self,text,ifname=ifname,sname=sname)

class NonExistingAttachment(LadonDispatchException):
	def __init__(self,ifname,sname,text):
		LadonDispatchException.__init__(self,text,ifname=ifname,sname=sname)

class InvalidAttachmentReference(LadonDispatchException):
	def __init__(self,ifname,sname,text):
		LadonDispatchException.__init__(self,text,ifname=ifname,sname=sname)



# -*- coding: utf-8 -*-

from ladon.compat import PORTABLE_STRING

catalog_default_template = PORTABLE_STRING("""
<!DOCTYPE html>
<html>
	<head>
		<meta content="text/html; charset={{ charset }}" http-equiv="Content-Type" />
		<title>{{ catalog_name }}</title>
		<style>
{{ css }}
{{ extra_css }}
		</style>
	</head>
	<body>
		<div class="catName">{{ catalog_name }}</div>
		<div class="catDesc">{{ catalog_desc }}</div>
		<div class="catContent">
			<ul class="catService">
{% for service in services %}
				<li>
					<b><a href="{{ client_path }}/{{ service.servicename }}{{ '?' if query_string }}{{query_string}}">{{ service.servicename }}</a></b><br/>
					{{ service.doc }}
				</li>
{% endfor %}
			</ul>
		</div>
		<div class="catGen">Powered by Ladon for Python</div>
	</body>
</html>
""")

service_default_template = PORTABLE_STRING("""
<!DOCTYPE html>
<html>
	<head>
		<meta content="text/html; charset={{ charset }}" http-equiv="Content-Type" />
		<title>{{ servicename }}</title>
		<style>
{{ css }}
{{ extra_css }}
		</style>
	</head>
	<body>
		<div class="service-header">
			<div class="service-title">{{ servicename }}</div>
			<form method="get" class="skin-selector">
				<label for="skin-select">skins:</label>
				<select id="skin-select" name="skin" onchange="document.forms[0].submit()">
					<option value="">Default</option>
					{% for skin in skins %}
					<option value="{{skin}}"{% if skin == current_skin %} selected{% endif %}>{{ skin|title }}</option>
					{% endfor %}
				</select>
			</form>
		</div>
		<div class="service-overview">
			<div class="headline">Overview</div>
			<div class="title">Methods</div>
			<ul class="list">
{% for method in methods %}
				<li>
					<a href="#{{ method.methodname }}"><span class="entry">{{ method.methodname }}</span></a>
					( )
				</li>
{% endfor %}
			</ul>
			<div class="title">Types</div>
			<ul class="list">
{% for type in types %}
				<li>
					<a href="#{{ type.name }}"><span class="entry">{{ type.name }}</span></a>
				</li>
{% endfor %}
			</ul>
		</div>

		<div class="service-description">
			<div class="title">Description</div>
			<div class="doc-lines">
				{{ doc }}
			</div>
		</div>
		<div class="service-interfaces">
			<div class="title">Interfaces</div>
			<ul class="list">
{% for interface in interfaces %}
				<li>{{ interface }} [ <a href="{{ client_path }}/{{ interface }}">url</a> <a href="{{ client_path }}/{{ interface }}/description">description</a> ]</li>
{% endfor %}
			</ul>
		</div>
		<div class="service-api">
			<div class="methods">
				<div class="title">Methods</div>
				<ul class="list">
	{% for method in methods %}
					<li class="entry">
						<div class="declaration">
							<a name="{{ method.methodname }}"></a><span class="name">{{ method.methodname }}</span>
							(
		{% set sep = '' %}
		{% for param in method.params %}
							{{ sep }} 
							<span class="param-type">
			{% if param.ladontype %}
								<a href="#{{ param.ladontype }}">{{ param.type }}</a>
			{% else %}
								{{ param.type }}
			{% endif %}
							</span> 
							<span class="param-name">{{ param.name }}</span>
			{% set sep = ',' %}
		{% endfor %}
							)
						</div>
						<div class="doc-lines">
							{{ method.doc }}
						</div>
		{% for param in method.params %}
			{% if loop.first %}
						<div class="params-title">Parameters</div>
						<ul class="params">
			{% endif %}
							<li>
								<span class="param-name">{{ param.name }}</span>: <span class="param-type">
			{% if param.ladontype %}
									<a href="#{{ param.ladontype }}">{{ param.type }}</a>
			{% else %}
									{{ param.type }}
			{% endif %}
								</span>
			{% if param.optional %}
								[ default: {{ param.default }} ]
			{% endif %}
								<br/>
								<div class="doc-lines">
									{{ param.doc }}
								</div>
							</li> 
			{% if loop.last %}
						</ul>
			{% endif %}
		{% endfor %}
						<div class="return-type-title">Return value</div>
						<div class="return-type">
							<span class="param-type">
		{% if method.returns.ladontype %}
								<a href="#{{ method.returns.ladontype }}">{{ method.returns.type }}</a>
		{% else %}
								{{ method.returns.type }}
		{% endif %}
							</span>
							<div class="doc-lines">
								{{ method.returns.doc }}
							</div>
						</div>
					</li>
	{% endfor %}
				</ul>
			</div>
			<div class="types">
				<div class="title">Types</div>
				<ul class="list">
	{% for type in types %}
					<li class="entry">
						<div class="definition">
							<a name="{{ type.name }}"></a><span class="name">{{ type.name }}</span>
						</div>
						<div class="attributes-title">Attributes</div>
						<ul class="attributes">
		{% for k,v in type.attributes.items() %}
							<li>
								<span class="param-name">{{ k }}</span>:
								<span class="param-type">
			{% if v.ladontype %}
									<a href="#{{v.ladontype}}">{{ v.type }}</a>
			{% else %}
									{{ v.type }}
			{% endif %}
								</span>
			{% set prefix = '[' %}
			{% set suffix = '' %}
			{% if 'nullable' in v.props and v.props.nullable==True %}
								{{ prefix }} Nullable
				{% set prefix = ' |' %}
				{% set suffix = ' ]' %}
			{% endif %}
			{% if 'default' in v.props %}
								{{ prefix }} default: {{ v.props.default }}
				{% set prefix = ' | ' %}
				{% set suffix = ' ]' %}
			{% endif %}
			{{ suffix }}
			{% if 'doc' in v.props %}
								<br/>
								<div class="doc-lines">
									{{ v.props.doc|join('\\n') }}
								</div>
			{% endif %}
							</li>
		{% endfor %}
						</ul>
					</li>
	{% endfor %}
				</ul>
			</div>
		</div>
		<div class="service-footer">Powered by Ladon for Python</div>
	</body>
</html>
""")



import sys
from ladon.types.attachment import attachment

if sys.version_info[0]==2:
	import StringIO,cStringIO
	StringIO = StringIO.StringIO
	BytesIO = cStringIO.StringIO
	PORTABLE_BYTES = str
	PORTABLE_STRING = unicode
	pytype_support = [int,long,str,unicode,bool,float]
	
	safe_conversions = {
		int: [int,long,str,unicode,float],
		long: [long,int,str,unicode], # Python 2 automatically turns int into long therefore this is regarded as a safe conversion
		str: [str,unicode],
		unicode: [unicode,str],
		bool: [bool,int,long,str,unicode,bool,float],
		float: [float,str,unicode]
	}
	
	PORTABLE_STRING_TYPES = [unicode,str]
	
	type_to_xsd = {
		int: 'long',
		long: 'long',
		str: 'string',
		unicode: 'string',
		bool: 'boolean',
		float: 'decimal',
		attachment: 'binary'
	}

	type_to_jsontype = {
		int: 'number',
		long: 'number',
		str: 'string',
		unicode: 'string',
		bool: 'boolean',
		float: 'float',
		attachment: 'attachment'
	}

elif sys.version_info[0]>=3:
	import io
	StringIO = io.StringIO
	BytesIO = io.BytesIO
	
	PORTABLE_BYTES = bytes
	PORTABLE_STRING = str
	pytype_support = [int,bytes,str,bool,float]

	safe_conversions = {
		int: [int,bytes,str],
		bytes: [bytes,str],
		str: [str,bytes],
		bool: [bool,int,bytes,str,bool,float],
		float: [float,bytes,str]
	}
	
	PORTABLE_STRING_TYPES = [str,bytes]
	
	type_to_xsd = {
		int: 'long',
		bytes: 'string',
		str: 'string',
		bool: 'boolean',
		float: 'decimal',
		attachment: 'binary'
	}

	type_to_jsontype = {
		int: 'number',
		bytes: 'string',
		str: 'string',
		bool: 'boolean',
		float: 'float',
		attachment: 'attachment'
	}



# -*- coding: utf-8 -*-

from ladon.ladonizer.collection import global_service_collection
from ladon.types import get_type_info,validate_type
from ladon.types.attachment import attachment,extract_attachment_reference
from ladon.types.typeconverter import TypeConverter
from ladon.exceptions.dispatcher import *
from ladon.exceptions.service import *
from ladon.tools.multiparthandler import AttachmentHandler
from ladon.server.customresponse import CustomResponse
from ladon.tools.log import debug
import os,re,traceback,time
from ladon.server import *

class Dispatcher(object):
	"""
	The dispatcher class handles the communication between the service interface and
	the user functionality. This happens in dispatch_request()
	
	1. It recieves the raw protocol data and attempts to parse it using the service
	interface's request handler. If successful a req_dict (request dictionary) is returned.

	2. The request dictionary is parsed to call_method() where the arguments are converted to
	fit the user defined method.
	
	3. If the call to the user defined method is successful the result is converted to a
	res_dict (response dictionary) in result_to_dict()
	
	4. The response dictionary is passed to the service interface's response handler and
	returned as raw protocol data.
	
	"""
	def __init__(self,sinst,ifclass,response_encoding,logging):
		self.response_encoding = response_encoding
		self.sinst = sinst
		self.iface = ifclass(sinst)
		self.logging = logging


	def call_method(self,method,req_dict,tc,export_dict,log_line):
		"""
		call_method converts the res_dict delivered from an interface
		to the type of arguments expected by the service method.
		tc is the TypeConverter associated to the service method.
		"""
		global rx_cid,rx_cidx
		args = []
		for arg in method.args():
			if arg['name'] not in req_dict['args']:
				if 'default' in arg:
					args += [arg['default']]
				else:
					raise UndefinedServiceMethod(self.iface._interface_name(),self.sinst.servicename,'Parameter "%s" is not optional' % arg['name'])
			else:
				if type(arg['type'])==list:
					arg_list = []
					type_info = get_type_info(arg['type'][0])
					if type_info:
						for item in req_dict['args'][arg['name']]:
							arg_list += [arg['type'][0](prime_dict=item,tc=tc,export_dict=export_dict)]
					elif arg['type'][0]==attachment:
						for item in req_dict['args'][arg['name']]:
							arg_list += [extract_attachment_reference(item,export_dict,self.response_encoding,self.iface._interface_name(),self.sinst.servicename)]
					else:
						for item in req_dict['args'][arg['name']]:
							arg_list += [tc.from_unicode_string(item,arg['type'][0])]
					args += [arg_list]
				else:
					type_info = get_type_info(arg['type'])
					val = req_dict['args'][arg['name']]
					if type_info:
						args += [arg['type'](prime_dict=val,tc=tc,export_dict=export_dict)]
					elif arg['type']==attachment:
						args += [extract_attachment_reference(val,export_dict,self.response_encoding,self.iface._interface_name(),self.sinst.servicename)]
					else:
						args += [tc.from_unicode_string(val,arg['type'])]
					
		#path,fname = os.path.split(method.sinfo.sourcefile)
		#mname = os.path.splitext(fname)[0]
		mname = method.sinfo.modulename
		service_module = __import__(mname, globals(),  locals(), '*')
		#file, pathname, description = imp.find_module(mname,[path])
		#service_module = imp.load_module(mname,file, pathname, description)
		service_class_instance = getattr(service_module,method.sinfo.servicename)()
		if self.logging & LOG_REQUEST_ACCESS:
			log_line += ['Method:%s.%s' % (method.sinfo.servicename,req_dict['methodname'])]
		if self.logging & LOG_REQUEST_DICT:
			log_line += ['RequestDict:%s' % (str(req_dict))]
		if self.logging & LOG_EXECUTION_TIME:
			start = time.time()
		if method._has_keywords:
			kw = {'LADON_METHOD_TC':tc}
			kw.update(export_dict)
			result = getattr(service_class_instance,req_dict['methodname'])(*args,**kw)
		else:
			result = getattr(service_class_instance,req_dict['methodname'])(*args)
		if self.logging & LOG_EXECUTION_TIME:
			log_line.insert(0,'ExecutionTime:%s' % str(time.time()-start))
		return result


	def result_to_dict(self,method,result,tc,response_attachments,log_line):
		"""
		Convert the result of a method call to it's dictionary representation.
		tc is a TypeConverter
		"""
		res_dict = {
			'servicename': method.sinfo.servicename,
			'servicenumber': method.sinfo.servicenumber,
			'method': method.name()}
		typ = method._rtype
		type_info = get_type_info(typ)
		if type_info==None:
			if [list,tuple].count(type(typ)):
				result_list = []
				res_dict['result'] = result_list
				type_info = get_type_info(typ[0])
				if result == typ:
					# Assumption list attributes are always optional
					return
				
				if type_info:
					for item in result:
						result_list += [item.__dict__(tc,response_attachments)]
				elif typ[0]==attachment:
					for item in result:
						if not type(item) == attachment:
							raise AttachmentExpected(self.iface._interface_name(),self.sinst.servicename,'Attachment expected got: %s' % type(item))
						result_list += [response_attachments.add_attachment(item)]
				else:
					for item in result:
						result_list += [tc.to_unicode_string(item,typ[0])]
			elif typ==attachment:
				res_dict['result'] = response_attachments.add_attachment(result)
			else:
				res_dict['result'] = tc.to_unicode_string(result,typ)
		else:
			res_dict['result'] = result.__dict__(tc,response_attachments)
		
		if self.logging & LOG_RESPONSE_DICT:
			log_line += ['ResponseDict:%s' % (str(res_dict))]
		return res_dict


	def dispatch_request(self,request_data,export_dict):
		log_line = []
		try:
			export_dict['response_attachments'] = AttachmentHandler()
			methodname,method = None,None
			req_dict = self.iface.parse_request(request_data,encoding=self.response_encoding)
			methodname = req_dict['methodname']
			method = self.sinst.method(methodname)
			if not method:
				raise UndefinedServiceMethod(self.iface._interface_name(),self.sinst.servicename,'Service method "%s" is not declared in service' % methodname)
			tc = TypeConverter(
				encoding=method._encoding,
				allow_unsafe_conversion=method._allow_unsafe_conversion,
				only_strings_to_unicode=(not self.iface.stringify_res_dict()))
			result = self.call_method(method,req_dict,tc,export_dict,log_line)
		except Exception as e:
			reflection = req_dict.get('mirror',None)
			if isinstance(e,ServiceFault):
				response = self.iface.build_fault_response(e,methodname,encoding=self.response_encoding,reflection=reflection)
			elif method==None:
				response = self.iface.build_fault_response(ClientFault(str(e)),methodname,encoding=self.response_encoding,reflection=reflection)
			else:
				response = self.iface.build_fault_response(ServerFault(str(e)),methodname,encoding=self.response_encoding,reflection=reflection)
			return response
		if isinstance(result,CustomResponse):
			# In some cases it can be nessecary to override the normal response system and return
			# something completely different - ie. 
			# 1. if a certain method should return a file as a http attachment response for
			#    browsers (Content-Disposition: attachment;).
			# 2. or a commandline tool that sends a SOAP request and should output raw text as
			#    result
			# Objects of CustomResponse descendents are intercepted in the wsgi_application part
			# so the service developer has full control over response headers and data.
			return result
		res_dict = self.result_to_dict(method,result,tc,export_dict['response_attachments'],log_line=log_line)
		if 'mirror' in req_dict:
			res_dict['reflection'] = req_dict['mirror']
		if 'id' in req_dict:
			res_dict['id'] = req_dict['id']
		response = self.iface.build_response(res_dict,encoding=self.response_encoding)
		if self.logging:
			debug('\t%s' % ('\t'.join(log_line)))
		return response



# -*- coding: utf-8 -*-
"""
Setup the loglevel for services on this particular server.

When the system calles the log function write() it is always associated with a
log-level. Whether or not the log-line reaches all the way to the syslog depends
on the current loglevel configuration.

The main log function is called write(). It logs by default in level 3 if not
overridden with the loglevel keyword argument.

Log-levels::
	Error levels:
	1 = Critical error (unimplemented resolution and undefined consequences)
	2 = Elevated error (high priority error)
	3 = Normal error
	
	Info level:
	4 = Info           (Log brief remarks about successful operations)
	
	Debug levels:
	5 = Notices        (inline log messages for tracing problems)
	6 = Debug          (manual tracing)
	7 = trace          (trace details)
"""

import os,logging,logging.handlers

__author__ = "Jakob Simon-Gaarde <jakob@simon-gaarde.dk>"

cur_loglevel = 4

def set_loglevel(level):
	global cur_loglevel
	cur_loglevel = level

def get_loglevel():
	global cur_loglevel
	return cur_loglevel

cur_logfile = '/tmp/ladon.log'
def set_logfile(logfile):
	global cur_logfile
	cur_logfile = logfile

def write(msg,loglevel=3,context=None,force=False):
	
	"""
	Maybe write a message to the syslog. If no loglevel is passed the default level is 3.
	
	Log-levels::
		Error levels:
		1 = Critical error (unimplemented resolution and undefined consequences)
		2 = Elevated error (high priority error)
		3 = Normal error
		
		Info level:
		4 = Info           (Log brief remarks about successful operations)
		
		Debug levels:
		5 = Notices        (inline log messages for tracing problems)
		6 = Debug          (manual tracing)
		7 = Trace          (trace details)
	"""
	global cur_loglevel,cur_logfile

	if cur_loglevel<loglevel and force==False:
		# Log level filters out this message
		return

	try:
		remote_host = os.environ['LADON_REMOTE_HOST']
		LOG_FILENAME = '%s.%s' % (cur_logfile,remote_host)
		logger = logging.getLogger('ladonlogger.%s' % remote_host)
	except:
		LOG_FILENAME = cur_logfile
		logger = logging.getLogger('ladonlogger')

	logger.setLevel(logging.DEBUG)

	# Add the log message handler to the logger
	if len(logger.handlers)==0:
		handler = logging.handlers.RotatingFileHandler(LOG_FILENAME, maxBytes=4000000, backupCount=5)
		formatter = logging.Formatter("%(asctime)s - %(message)s")
		handler.setFormatter(formatter)
		logger.addHandler(handler)

	logger.debug(msg)


def info(msg,context=None,force=False):
	"""
	Send a brief remark to the syslog upon successful operations. Info will only reach the syslog if the current
	loglevel configuration is set to 4 or more.
	"""
	write(msg,4,context,force)


def notice(msg,context=None,force=False):
	"""
	Send a notice to the syslog. Notices will only reach the syslog if the current
	loglevel configuration is set to 5 or more.
	"""
	write(msg,5,context,force)

def debug(msg,context=None,force=False):
	"""
	Send a debug message to the syslog. Debug messages will only reach the syslog if the current
	loglevel configuration is set to 6 or more.
	"""
	write(msg,6,context,force)

def trace(msg,context=None,force=False):
	"""
	Send a debug message to the syslog. Debug messages will only reach the syslog if the current
	loglevel configuration is set to 6 or more.
	"""
	write(msg,7,context,force)



# -*- coding: utf-8 -*-

"""
This module contains the entry-point for all Ladon Service methods.
Every method that should be exposed to the interfaces supported in
a given Ladon installation must register itself through the ladonize
decorator.
"""

__author__ = "Jakob Simon-Gaarde"

from ladon.ladonizer.collection import global_service_collection
from ladon.types import validate_type,result_to_dict
from ladon.exceptions.ladonizer import *
import ladon.tools.log as log
import time

def ladonize(*def_args,**def_kw):
	"""
	The entry-point for all Ladon service methods. You could call
	this the magic-decorator that does initiates all the parsing
	analysing and pre-storing of service method capabilities and
	requirements.
	The ladonize decorator takes exactly the same amount of
	arguments as the method it is decorating plus one mandatory
	keyword argument *rtype* that type-defines the return type of
	the method. 
	Each argument given to the decorator type-defines the order-
	wise corresponding parameter of the method being decorated::
		
		class SessionService(object):
		
			@ladonize(str,str,rtype=str)
			def login(self,username,password):
				...
				...
				return session_id

	There are a couple of optional keyword arguments to ladonize
	aswell:
		+-------------------------+------------------------------------------------------------------------------------+
		| keyword arg             | Description                                                                        |
		+=========================+====================================================================================+
		| encoding                | Control the encoding to be used internally in the service method for raw strings.  |
		+-------------------------+------------------------------------------------------------------------------------+
		| allow_unsafe_conversion | True means that Ladon will try to convert arguments and return values if nessecary |
		+-------------------------+------------------------------------------------------------------------------------+
	"""
	def decorator(f):
		def injector(*args,**kw):
			"""
			The Ladon inner injection function is called from the dispatcher. It does run-time type
			checking against the types registered with the service method. If everything is OK the
			user method is called. The result of the userspace-method is then checked before it is
			passed back to the dispatcher.
			"""
			
			# Get the LadonMethodInfo object generated at parsetime which is stored as a member
			# on the function object
			lmi = injector._ladon_method_info
			# Reference the incomming arguments in callargs (filter out the function reference)
			callargs = args[1:]
			
			for argidx in range(len(callargs)):
				# Check the type of each argument against the type which the method has been 
				# registered to take in the order-wise corresponding argument
				if not validate_type(lmi._arg_types[argidx],callargs[argidx]):
					# Raise ArgTypeMismatch
					raise ArgTypeMismatch(
						lmi.sinfo,
						lmi._func_name,
						lmi._arg_names[argidx],
						lmi._arg_types[argidx],
						type(callargs[argidx]))
			
			# Call the userspace service method (**kw will be used to transport Ladon info
			# and tools all the way to userspace of the service method. I.e. the TypeConverter
			# is passed with the keyword "LADON_METHOD_TC")
			debug_mode = log.get_loglevel()>=6
			if debug_mode:
				log_line = [
					'Method:%s.%s' % (lmi.sinfo.servicename,lmi._func_name),
					'RequestPositionalArgs:%s' % str(args[1:]),
					'RequestKeywordArgs:%s' % str(kw)]
			
			if debug_mode:
				try:
					start = time.time()
					res = f(*args,**kw)
					log_line.insert(0,'ExecutionTime:%s' % str(time.time()-start))
				except Exception as e:
					log_line += ['Exception:%s' % str(e) ]
					log.debug('\t%s' % ('\t'.join(log_line)))
					raise e
				
				log_line += [ 'ReturnValue:%s' % str(result_to_dict(lmi,res)) ]
				log.debug('\t%s' % ('\t'.join(log_line)))
				
			else:
				res = f(*args,**kw)
			# Check the return type
			if not validate_type(lmi._rtype,res):
				# Raise Arg-type mismatch 
				raise ReturnTypeMismatch(lmi.sinfo,lmi._func_name,lmi._rtype,type(res))
			
			# Return the result to the dispatcher
			return res
		
		# Register the service method and all the types required by it 
		ladon_method_info = global_service_collection().add_service_method(f,*def_args,**def_kw)
		
		# store the LadonMethodInfo object directly on the fuction object
		injector._ladon_method_info = ladon_method_info
		injector.__doc__ = ladon_method_info._doc
		injector.func_name = ladon_method_info._func_name
		return injector
		
	return decorator 



# -*- coding: utf-8 -*-

from ladon.interfaces.base import BaseInterface,ServiceDescriptor,BaseRequestHandler,BaseResponseHandler,BaseFaultHandler
from ladon.interfaces import expose
from ladon.compat import type_to_jsontype,pytype_support,PORTABLE_STRING
import json,sys,traceback

class JSONWSPServiceDescriptor(ServiceDescriptor):
	
	javascript_type_map = type_to_jsontype
	version = '1.1'
	_content_type = 'application/json'

	def generate(self,servicename,servicenumber,typemanager,methodlist,service_url,encoding):
		type_dict = typemanager.type_dict
		type_order = typemanager.type_order

		def map_type(typ):
			if typ in JSONWSPServiceDescriptor.javascript_type_map:
				return JSONWSPServiceDescriptor.javascript_type_map[typ]
			else:
				return typ.__name__

		desc = {
			'servicename': servicename,
			'url': service_url,
			'type': 'jsonwsp/description',
			'version': self.version,
			'types': {},
			'methods': {}
		}
		
		types = desc['types']
		for typ in type_order:
			if type(typ)==dict:
				desc_type = {}
				types[typ['name']] = desc_type
				for k,v,props in typ['attributes']:
					if type(v)==list:
						desc_type_val = [map_type(v[0])]
					else:
						desc_type_val = map_type(v)
					
					if self.version>'1.0':
						attr_props = dict(props)
						attr_props['type'] = desc_type_val
						desc_type[k] = attr_props
					else:
						desc_type[k] = desc_type_val

		methods = desc['methods']
		for m in methodlist:
			desc_mparams = {}
			order = 1
			desc_method = {'params': desc_mparams, 'doc_lines': m._method_doc}
			methods[m.name()] = desc_method
			for arg in m.args():
				if [list,tuple].count(type(arg['type'])):
					desc_param_type = [map_type(arg['type'][0])]
				else:
					desc_param_type = map_type(arg['type'])
				desc_mparams[arg['name']] = {
					"type": desc_param_type,
					"def_order": order,
					"optional": arg['optional'],
					}
				if 'doc' in arg:
					desc_mparams[arg['name']]["doc_lines"] = arg['doc']
				else:
					desc_mparams[arg['name']]["doc_lines"] = []
				if 'default' in arg:
					desc_mparams[arg['name']]["default"] = arg['default']

				order += 1
				
			if [list,tuple].count(type(m._rtype)):
				desc_rtype = [map_type(m._rtype[0])]
			else:
				desc_rtype = map_type(m._rtype)
			desc_method['ret_info'] = {
				'type': desc_rtype,
				'doc_lines': m._rtype_doc
			}
			
		if sys.version_info[0]>=3:
			return json.dumps(desc)
		return json.dumps(desc,encoding=encoding)



class JSONWSPRequestHandler(BaseRequestHandler):
	
	def parse_request(self,json_body,sinfo,encoding):
		def parse_number(x):
			return PORTABLE_STRING(x)
		def parse_constant(x):
			#if x=='false':
				#return PORTABLE_STRING("False")
			#elif x=='true':
				#return PORTABLE_STRING("True")
			if x=='null':
				return PORTABLE_STRING("None")
			return PORTABLE_STRING(x)
		req_dict = json.loads(PORTABLE_STRING(json_body,encoding),parse_int=parse_number,parse_float=parse_number,parse_constant=parse_constant)
		return req_dict



class JSONWSPResponseHandler(BaseResponseHandler):
	
	_content_type = 'application/json'
	_stringify_res_dict = False
	version = '1.0'
	
	def build_response(self,res_dict,sinfo,encoding):
		res_dict['type'] = 'jsonwsp/response'
		res_dict['version'] = self.version
		res_dict['methodname'] = res_dict['method']
		del res_dict['method']
		return json.dumps(res_dict,ensure_ascii=False).encode(encoding)

class JSONWSPFaultHandler(BaseFaultHandler):
	
	_content_type = 'text/xml'
	_stringify_res_dict = False
	version = '1.0'
	
	def build_fault_response(self,service_exc,sinfo,methodname,encoding,reflection):
		if service_exc.detail:
			detail = service_exc.detail
		else:
			detail = traceback.format_exc()
		detail = detail.replace('\r\n','\n').split('\n')
		fault_dict = {
			'type': 'jsonwsp/fault',
			'version': self.version,
			'fault': {
				'code': service_exc.faultcode,
				'string': service_exc.faultstring,
				'detail': detail,
				'filename': service_exc.mod,
				'lineno': service_exc.lineno
			}
		}
		if reflection:
			fault_dict['reflection'] = reflection
		return json.dumps(fault_dict,ensure_ascii=False).encode(encoding)


@expose
class JSONWSPInterface(BaseInterface):

	def __init__(self,sinfo,**kw):
		def_kw = {
			'service_descriptor': JSONWSPServiceDescriptor,
			'request_handler': JSONWSPRequestHandler,
			'response_handler': JSONWSPResponseHandler,
			'fault_handler': JSONWSPFaultHandler}
		def_kw.update(kw)
		BaseInterface.__init__(self,sinfo,**def_kw)

	@staticmethod
	def _interface_name():
		return 'jsonwsp'

	@staticmethod
	def _accept_basetype(typ):
		return pytype_support.count(typ)>0

	@staticmethod
	def _accept_list():
		return True

	@staticmethod
	def _accept_dict():
		return False




######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Communicator client code.
# 
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998
# the Initial Developer. All Rights Reserved.
# 
# Contributor(s):
#   Mark Pilgrim - port to Python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from . import constants
import sys
from .charsetprober import CharSetProber

class CharSetGroupProber(CharSetProber):
    def __init__(self):
        CharSetProber.__init__(self)
        self._mActiveNum = 0
        self._mProbers = []
        self._mBestGuessProber = None
        
    def reset(self):
        CharSetProber.reset(self)
        self._mActiveNum = 0
        for prober in self._mProbers:
            if prober:
                prober.reset()
                prober.active = True
                self._mActiveNum += 1
        self._mBestGuessProber = None

    def get_charset_name(self):
        if not self._mBestGuessProber:
            self.get_confidence()
            if not self._mBestGuessProber: return None
#                self._mBestGuessProber = self._mProbers[0]
        return self._mBestGuessProber.get_charset_name()

    def feed(self, aBuf):
        for prober in self._mProbers:
            if not prober: continue
            if not prober.active: continue
            st = prober.feed(aBuf)
            if not st: continue
            if st == constants.eFoundIt:
                self._mBestGuessProber = prober
                return self.get_state()
            elif st == constants.eNotMe:
                prober.active = False
                self._mActiveNum -= 1
                if self._mActiveNum <= 0:
                    self._mState = constants.eNotMe
                    return self.get_state()
        return self.get_state()

    def get_confidence(self):
        st = self.get_state()
        if st == constants.eFoundIt:
            return 0.99
        elif st == constants.eNotMe:
            return 0.01
        bestConf = 0.0
        self._mBestGuessProber = None
        for prober in self._mProbers:
            if not prober: continue
            if not prober.active:
                if constants._debug:
                    sys.stderr.write(prober.get_charset_name() + ' not active\n')
                continue
            cf = prober.get_confidence()
            if constants._debug:
                sys.stderr.write('%s confidence = %s\n' % (prober.get_charset_name(), cf))
            if bestConf < cf:
                bestConf = cf
                self._mBestGuessProber = prober
        if not self._mBestGuessProber: return 0.0
        return bestConf
#        else:
#            self._mBestGuessProber = self._mProbers[0]
#            return self._mBestGuessProber.get_confidence()



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Universal charset detector code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#   Shy Shalom - original C code
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from . import constants
import sys
from .charsetgroupprober import CharSetGroupProber
from .sbcharsetprober import SingleByteCharSetProber
from .langcyrillicmodel import Win1251CyrillicModel, Koi8rModel, Latin5CyrillicModel, MacCyrillicModel, Ibm866Model, Ibm855Model
from .langgreekmodel import Latin7GreekModel, Win1253GreekModel
from .langbulgarianmodel import Latin5BulgarianModel, Win1251BulgarianModel
from .langhungarianmodel import Latin2HungarianModel, Win1250HungarianModel
from .langthaimodel import TIS620ThaiModel
from .langhebrewmodel import Win1255HebrewModel
from .hebrewprober import HebrewProber

class SBCSGroupProber(CharSetGroupProber):
    def __init__(self):
        CharSetGroupProber.__init__(self)
        self._mProbers = [ \
            SingleByteCharSetProber(Win1251CyrillicModel),
            SingleByteCharSetProber(Koi8rModel),
            SingleByteCharSetProber(Latin5CyrillicModel),
            SingleByteCharSetProber(MacCyrillicModel),
            SingleByteCharSetProber(Ibm866Model),
            SingleByteCharSetProber(Ibm855Model),
            SingleByteCharSetProber(Latin7GreekModel),
            SingleByteCharSetProber(Win1253GreekModel),
            SingleByteCharSetProber(Latin5BulgarianModel),
            SingleByteCharSetProber(Win1251BulgarianModel),
            SingleByteCharSetProber(Latin2HungarianModel),
            SingleByteCharSetProber(Win1250HungarianModel),
            SingleByteCharSetProber(TIS620ThaiModel),
            ]
        hebrewProber = HebrewProber()
        logicalHebrewProber = SingleByteCharSetProber(Win1255HebrewModel, False, hebrewProber)
        visualHebrewProber = SingleByteCharSetProber(Win1255HebrewModel, True, hebrewProber)
        hebrewProber.set_model_probers(logicalHebrewProber, visualHebrewProber)
        self._mProbers.extend([hebrewProber, logicalHebrewProber, visualHebrewProber])

        self.reset()



# -*- coding: utf-8 -*-

from ladon.interfaces.base import BaseInterface,ServiceDescriptor,BaseRequestHandler,BaseResponseHandler,BaseFaultHandler
from ladon.interfaces import expose
from ladon.compat import PORTABLE_STRING,type_to_xsd,pytype_support,BytesIO
from xml.sax.handler import ContentHandler,feature_namespaces
from xml.sax import make_parser
from xml.sax.xmlreader import InputSource
import sys,re,traceback

rx_nil_attr = re.compile(PORTABLE_STRING('^\w*[:]{0,1}nil$'),re.I)

class SOAP11ServiceDescriptor(ServiceDescriptor):
	
	xsd_type_map = type_to_xsd
	_content_type = 'text/xml'
	
	def generate(self,servicename,servicenumber,typemanager,methodlist,service_url,encoding):
		"""
		Generate WSDL file for SOAP11Interface
		"""
		type_dict = typemanager.type_dict
		type_order = typemanager.type_order

		def map_type(typ):
			if typ in SOAP11ServiceDescriptor.xsd_type_map:
				return SOAP11ServiceDescriptor.xsd_type_map[typ]
			else:
				return typ.__name__

		import xml.dom.minidom as md
		doc = md.Document()
		
		# SERVICE DEFINITION
		# Create the definitions element for the service
		definitions = doc.createElement('wsdl:definitions')
		definitions.setAttribute('xmlns:mime','http://schemas.xmlsoap.org/wsdl/mime/')
		definitions.setAttribute('xmlns:soap','http://schemas.xmlsoap.org/wsdl/soap/')
		definitions.setAttribute('xmlns:http','http://schemas.xmlsoap.org/wsdl/http/')
		definitions.setAttribute('xmlns:wsdl','http://schemas.xmlsoap.org/wsdl/')
		definitions.setAttribute('name', servicename)
		definitions.setAttribute('targetNamespace','urn:%s' % servicename)
		definitions.setAttribute('xmlns:tns','urn:%s' % servicename)
		definitions.setAttribute('xmlns:soapenc','http://schemas.xmlsoap.org/soap/encoding/')
		definitions.setAttribute('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance')
		definitions.setAttribute('xmlns:xsd','http://www.w3.org/2001/XMLSchema')
		definitions.setAttribute('xmlns:ns%d' % servicenumber,'urn:%s' % servicename)
		doc.appendChild(definitions)
		
		# TYPES
		# The types element
		types = doc.createElement('wsdl:types')
		definitions.appendChild(types)
		
		# Service schema for types required by the target namespace we defined in the definition element
		schema = doc.createElement('xsd:schema')
		schema.setAttribute('targetNamespace','urn:%s' % servicename)
		schema.setAttribute('xmlns:ns%d' % servicenumber,'urn:%s' % servicename)
		types.appendChild(schema)
		
		# Import namespace schema
		import_tag = doc.createElement('xsd:import')
		import_tag.setAttribute('namespace','http://schemas.xmlsoap.org/soap/encoding/')
		schema.appendChild(import_tag)

		# Define types, the type_order variable holds all that need to be defined and in the
		# correct order.
		# * If a list is encountered as a type it will be handled as a complex type with a single element reflecting the inner type.
		# * LadonTypes (identified by being contained in type_dict) are also handled as complex types with an element-tag per attribute
		# * Primitive types (either as LadonType attributes or list inner-types) are added as xsd - SOAP types.
		for typ in type_order:
			if isinstance(typ, list):
				inner = typ[0]
				complextype = doc.createElement('xsd:complexType')
				complextype.setAttribute('name','ArrayOf%s' % inner.__name__)
				schema.appendChild(complextype)

				# Old
				#complexcontent = doc.createElement('xsd:complexContent')
				#complextype.appendChild(complexcontent)
				#restriction = doc.createElement('xsd:restriction')
				#restriction.setAttribute('base','soapenc:Array')
				#complexcontent.appendChild(restriction)
				##sequence = doc.createElement('xsd:sequence')
				##element = doc.createElement('xsd:element')
				##element.setAttribute('name','item')
				##if inner in type_dict:
					##element.setAttribute('type','ns%d:%s' % (servicenumber,inner.__name__))
				##else:
					##element.setAttribute('type','xsd:%s' % map_type(inner))
				##element.setAttribute('minOccurs','0')
				##element.setAttribute('maxOccurs','unbounded')
				##sequence.appendChild(element)
				##restriction.appendChild(sequence)
				#attribute = doc.createElement('xsd:attribute')
				#attribute.setAttribute('ref','soapenc:arrayType')
				#if inner in type_dict:
					#attribute.setAttribute('wsdl:arrayType','ns%d:%s[]' % (servicenumber,inner.__name__))
				#else:
					#attribute.setAttribute('wsdl:arrayType','xsd:%s[]' % map_type(inner))
				#restriction.appendChild(attribute)
				
				# New
				sequence = doc.createElement('xsd:sequence')
				element = doc.createElement('xsd:element')
				element.setAttribute('name','item')
				if inner in type_dict:
					element.setAttribute('type','ns%d:%s' % (servicenumber,inner.__name__))
				else:
					element.setAttribute('type','xsd:%s' % map_type(inner))
				element.setAttribute('minOccurs','0')
				element.setAttribute('maxOccurs','unbounded')
				element.setAttribute('nillable','true')
				sequence.appendChild(element)
				complextype.appendChild(sequence)
			else:
				complextype = doc.createElement('xsd:complexType')
				complextype.setAttribute('name',typ['name'])
				schema.appendChild(complextype)
				sequence = doc.createElement('xsd:sequence')
				complextype.appendChild(sequence)
				for k,v,props in typ['attributes']:
					element = doc.createElement('xsd:element')
					element.setAttribute('name',k.replace('_','-'))
					element.setAttribute('maxOccurs','1')
					element.setAttribute('minOccurs','1')
					if props.get('nullable')==True:
						element.setAttribute('minOccurs','0')
						element.setAttribute('nillable','true')
					if isinstance(v, list):
						inner = v[0]
						element.setAttribute('type','ns%d:ArrayOf%s' % (servicenumber,inner.__name__))
						element.setAttribute('minOccurs','0')
						element.setAttribute('nillable','true')
					else:
						if v in type_dict:
							element.setAttribute('type','ns%d:%s' % (servicenumber,v.__name__))
						else:
							element.setAttribute('type','xsd:%s' % map_type(v))
					sequence.appendChild(element)

		#<complexType name="ArrayOfstring">
		#<complexContent>
		#<restriction base="SOAP-ENC:Array">
		#<sequence>
		#<element name="attr" type="xsd:string" minOccurs="0" maxOccurs="unbounded"/>
		#</sequence>
		#<attribute ref="SOAP-ENC:arrayType" WSDL:arrayType="xsd:string[]"/>
		#</restriction>
		#</complexContent>
		#</complexType>

		#<message name="listUserRoles">
		#<part name="session-id" type="xsd:string"/>
		#<part name="domain" type="xsd:string"/>
		#<part name="uid" type="xsd:string"/>
		#</message>

		#<message name="listUserRolesResponse">
		#<part name="capa-res" type="ns2:CapaResult"/>
		#<part name="roles" type="ns2:ArrayOfRoleInfo"/>
		#</message>

		for m in methodlist:
			message = doc.createElement('wsdl:message')
			message.setAttribute('name',m.name())
			definitions.appendChild(message)
			for arg in m.args():
				part = doc.createElement('wsdl:part')
				part.setAttribute('name',arg['name'].replace('_','-'))
				if isinstance(arg['type'], (list, tuple)):
					part.setAttribute('type','ns%d:ArrayOf%s' % (servicenumber,arg['type'][0].__name__))
				else:
					if arg['type'] in type_dict:
						part.setAttribute('type','ns%d:%s' % (servicenumber,arg['type'].__name__))
					else:
						part.setAttribute('type','xsd:%s' % map_type(arg['type']))
				message.appendChild(part)
			message = doc.createElement('wsdl:message')
			message.setAttribute('name',"%sResponse" % m.name())
			definitions.appendChild(message)
			if isinstance(m._rtype, (list, tuple)):
				part = doc.createElement('wsdl:part')
				part.setAttribute('name','result')
				part.setAttribute('type','ns%d:ArrayOf%s' % (servicenumber,m._rtype[0].__name__))
				message.appendChild(part)
			elif m._rtype in type_dict:
				part = doc.createElement('wsdl:part')
				part.setAttribute('name','result')
				part.setAttribute('type','ns%d:%s' % (servicenumber,m._rtype.__name__))
				message.appendChild(part)
				#for k,v,props in type_dict[m._rtype]['attributes']:
					#part = doc.createElement('wsdl:part')
					#part.setAttribute('name',k.replace('_','-'))
					##part.setAttribute('maxOccurs','1')
					#if isinstance(v, list):
						#inner = v[0]
						#part.setAttribute('type','ns%d:ArrayOf%s' % (servicenumber,inner.__name__))
						##part.setAttribute('minOccurs','0')
						##part.setAttribute('nillable','true')
					#else:
						#if v in type_dict:
							#part.setAttribute('type','ns%d:%s' % (servicenumber,v.__name__))
						#else:
							#part.setAttribute('type','xsd:%s' % map_type(v))
						##part.setAttribute('minOccurs','1')
					#message.appendChild(part)
			else:
				part = doc.createElement('wsdl:part')
				part.setAttribute('name','result')
				part.setAttribute('type','xsd:%s' % map_type(m._rtype))
				message.appendChild(part)

		#<portType name="userservicePortType">
		#<operation name="createUser">
		#<documentation>Service definition of function ns2__createUser</documentation>
		#<input message="tns:createUser"/>
		#<output message="tns:createUserResponse"/>
		#</operation>
		porttype = doc.createElement('wsdl:portType')
		porttype.setAttribute('name','%sPortType' % servicename)
		definitions.appendChild(porttype)

		for m in methodlist:
			operation = doc.createElement('wsdl:operation')
			operation.setAttribute('name',m.name())
			porttype.appendChild(operation)
			if m.__doc__:
				documentation = doc.createElement('wsdl:documentation')
				documentation.appendChild(doc.createTextNode(m.__doc__))
				operation.appendChild(documentation)
			input_tag = doc.createElement('wsdl:input')
			input_tag.setAttribute('message','tns:%s' % m.name())
			operation.appendChild(input_tag)
			output_tag = doc.createElement('wsdl:output')
			output_tag.setAttribute('message','tns:%sResponse' % m.name())
			operation.appendChild(output_tag)



		#<binding name="userservice" type="tns:userservicePortType">
		#<SOAP:binding style="rpc" transport="http://schemas.xmlsoap.org/soap/http"/>
		#<operation name="createUser">
		#<SOAP:operation style="rpc" soapAction=""/>
		#<input>
		#<SOAP:body use="encoded" namespace="urn:userservice" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>
		#</input>
		#<output>
		#<SOAP:body use="encoded" namespace="urn:userservice" encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"/>
		#</output>
		#</operation>

		binding = doc.createElement('wsdl:binding')
		binding.setAttribute('name',servicename)
		binding.setAttribute('type',"tns:%sPortType" % servicename)
		transport = doc.createElement('soap:binding')
		transport.setAttribute('style','rpc')
		transport.setAttribute('transport','http://schemas.xmlsoap.org/soap/http')
		binding.appendChild(transport)
		definitions.appendChild(binding)

		for m in methodlist:
			operation = doc.createElement('wsdl:operation')
			operation.setAttribute('name',m.name())
			binding.appendChild(operation)
			soapaction = doc.createElement('soap:operation')
			soapaction.setAttribute('style','rpc')
			soapaction.setAttribute('soapAction',"%s/%s" % (service_url,m.name()))
			operation.appendChild(soapaction)
			
			m._multipart_response_required
			input_tag = doc.createElement('wsdl:input')
			body_parent = input_tag
			if m._multipart_request_required:
				multipart_related = doc.createElement('mime:multipartRelated')
				mime_body_part = doc.createElement('mime:part')
				body_parent = mime_body_part
				mime_content_part = doc.createElement('mime:part')
				mime_content = doc.createElement('mime:content')
				mime_content.setAttribute('type','*/*')
				input_tag.appendChild(multipart_related)
				multipart_related.appendChild(mime_body_part)
				multipart_related.appendChild(mime_content_part)
				mime_content_part.appendChild(mime_content)
				
			input_soapbody = doc.createElement('soap:body')
			input_soapbody.setAttribute('use','encoded')
			input_soapbody.setAttribute('namespace','urn:%s' % servicename)
			input_soapbody.setAttribute('encodingStyle','http://schemas.xmlsoap.org/soap/encoding/')
			body_parent.appendChild(input_soapbody)
			operation.appendChild(input_tag)
			output_tag = doc.createElement('wsdl:output')
			body_parent = output_tag
			if m._multipart_request_required:
				multipart_related = doc.createElement('mime:multipartRelated')
				mime_body_part = doc.createElement('mime:part')
				body_parent = mime_body_part
				mime_content_part = doc.createElement('mime:part')
				mime_content = doc.createElement('content:part')
				mime_content.setAttribute('type','*/*')
				output_tag.appendChild(multipart_related)
				multipart_related.appendChild(mime_body_part)
				multipart_related.appendChild(mime_content_part)
				mime_content_part.appendChild(mime_content)

			output_soapbody = doc.createElement('soap:body')
			output_soapbody.setAttribute('use','encoded')
			output_soapbody.setAttribute('namespace','urn:%s' % servicename)
			output_soapbody.setAttribute('encodingStyle','http://schemas.xmlsoap.org/soap/encoding/')
			body_parent.appendChild(output_soapbody)
			operation.appendChild(output_tag)


		#<service name="userservice">
		#<documentation>gSOAP 2.7.10 generated service definition</documentation>
		#<port name="userservice" binding="tns:userservice">
		#<SOAP:address location="http://127.0.0.1:8443/userservice"/>
		#</port>
		#</service>

		service = doc.createElement('wsdl:service')
		service.setAttribute('name',servicename)
		documentation = doc.createElement('wsdl:documentation')
		documentation.appendChild(doc.createTextNode('Ladon generated service definition'))
		service.appendChild(documentation)
		port = doc.createElement('wsdl:port')
		port.setAttribute('name',servicename)
		port.setAttribute('binding','tns:%s' % servicename)
		service.appendChild(port)
		address = doc.createElement('soap:address')
		address.setAttribute('location',service_url)
		port.appendChild(address)
		definitions.appendChild(service)
		if sys.version_info[0]>=3:
			return doc.toxml()
		return doc.toxml(encoding)


def u(instring):
	if sys.version_info[0]==2:
		return PORTABLE_STRING(instring,'utf-8')
	else:
		return PORTABLE_STRING(instring)

class ContainerSetRef(object):
	def __init__(self,c,refval):
		self.c = c
		self.refval = refval
	
	def set(self,val):
		self.c[self.refval] = val

class SOAP11ContentHandler(ContentHandler):
	# Define relevant XML parser states
	ENVELOPE_IN = 1
	HEADER_IN = 2
	BODY_IN = 3
	METHOD_IN = 4
	ARGS_IN = 5
	ARGS_OUT = 6

	def __init__(self,parser):
		"""
		Initialize
		"""
		self.parser = parser
		self.depth = 0
		self.method_depth = -1
		self.state = None
		self.req_dict = {'args':{}}
		self.current_arg_object = self.req_dict['args']
		self.arg_object_stack = []
		self.arg_tagname_stack = []
		self.prev_depth = None
		self.in_cdata = False
		self.multi_ref_hrefs = {}
		self.multi_ref_ids = {}
		self.cur_id_depth = None
		self.cur_id_val = None
		
	def startCdataSection(self):
		"""
		Mark that parser is in CDATA Section.
		"""
		self.in_cdata = True

	def endCdataSection(self):
		"""
		Mark that parser stepped out of a CDATA Section.
		"""
		self.in_cdata = False

	def startDocument(self):
		"""
		Setup XMLParser callbacks, callbacks are setup here as the XMLParser
		is not initialized by XMLReader before it starts parsing.
		"""
		self.parser._parser.StartCdataSectionHandler = self.startCdataSection
		self.parser._parser.EndCdataSectionHandler = self.endCdataSection

	def endDocument(self):
		for id_val,v in self.multi_ref_hrefs.items():
			if id_val in self.multi_ref_ids:
				v.set(self.multi_ref_ids[id_val])
	
	def startElement(self,name,attrs):
		"""
		Handle element entrance.
		"""
		# FIXME: qucik fix for name conversion
		is_null = False
		href_val,id_val = None,None
		for ak,av in attrs.items():
			if rx_nil_attr.match(ak) and av.lower()=='true':
				is_null = True
			elif ak.lower()=='href':
				href_val = av[1:]
			elif ak.lower()=='id':
				self.cur_id_val = av
				self.cur_id_depth = self.depth
				self.multi_ref_ids[av] = {}
				self.current_arg_object = self.multi_ref_ids[av]

		name = name.replace('-', '_')
		# Split element tag into tagname and prefix if nessecary.
		m = name.split(':')
		if len(m)>1:
			prefix,tagname = m
		else:
			prefix,tagname = [''] + m
		# Detect key positions in the SOAP envolope updating the parse pregress state.
		if not self.depth and tagname.lower()=='envelope':
			self.state = self.ENVELOPE_IN
		elif self.state == self.ENVELOPE_IN and tagname.lower()=='header':
			self.state = self.HEADER_IN
		elif self.state in [ self.HEADER_IN,self.ENVELOPE_IN ] and tagname.lower()=='body':
			self.state = self.BODY_IN
		elif self.state == self.BODY_IN:
			# Stepping into the method-name tag.
			self.state = self.METHOD_IN
			self.req_dict['methodname'] = tagname
			self.method_depth = self.depth
		elif self.state == self.METHOD_IN:
			# Stepping into the arguments section.
			self.state = self.ARGS_IN
		
		if self.state == self.ARGS_IN or self.cur_id_val:
			# Starting the argument parsing.
			self.pickup_content = u('')
			
			if is_null:
				newobject = None
			else:
				newobject = {}
			if tagname.lower() == 'item':
				# If the tagname "item" is intercepted it means that we are looking at an array.
				# Therefore the parent dict must be re-assigned to a list object and all objects
				# that follow (until the parser steps out of the item tag) be appended to this
				# list. Lists are ignored by the stack of objects and tagnames
				parent_tagname = self.arg_tagname_stack[-1:][0]
				parent_object = self.arg_object_stack[-1:][0]
				if isinstance(parent_object[parent_tagname], dict):
					# first child object
					parent_object[parent_tagname] = [newobject]
				else:
					# 2nd, 3rd ...
					parent_object[parent_tagname] += [newobject]
				if href_val:
					self.multi_ref_hrefs[href_val] = ContainerSetRef(parent_object,len(parent_object)-1)
			else:
				# Append to the stacks
				self.current_arg_object[tagname] = newobject
				self.arg_object_stack.append(self.current_arg_object)
				self.arg_tagname_stack.append(tagname)
				if href_val:
					self.multi_ref_hrefs[href_val] = ContainerSetRef(self.current_arg_object,tagname)
			self.current_arg_object = newobject
			self.prev_depth = self.depth
		self.depth += 1


	def endElement(self,name):
		# FIXME: qucik fix for name conversion
		name = name.replace('-', '_')
		self.depth -= 1
		# Split element tag into tagname and prefix if nessecary.
		m = name.split(':')
		if len(m)>1:
			prefix,tagname = m
		else:
			prefix,tagname = [''] + m
		# Stop collecting arguments when the parser steps out of the method-name tag.
		if self.depth==self.method_depth:
			# State ARGS_OUT means done parsing arguments.
			self.state = self.ARGS_OUT
		# In all other occations than tagname=='item' pop the stack of argument objects and tagnames.
		if self.depth==self.cur_id_depth:
			if len(self.multi_ref_ids[self.cur_id_val].items()[0][1]):
				self.multi_ref_ids[self.cur_id_val] = self.multi_ref_ids[self.cur_id_val].items()[0][1]
			else:
				self.multi_ref_ids[self.cur_id_val] = self.pickup_content
			self.cur_id_val = None
			self.cur_id_depth = None
		if self.state == self.ARGS_IN or self.cur_id_val:
			if not tagname.lower()=='item':
				if self.depth == self.prev_depth:
					# If parser is stepping out of the element it previously entered it means that
					# there were no child elements, therefore content in between becomes actual
					# argument data.
					parent_tagname = self.arg_tagname_stack[-1:][0]
					parent_object = self.arg_object_stack[-1:][0]
					if not parent_object[parent_tagname]==None:
						parent_object[parent_tagname] = self.pickup_content
				# pop stack of argument objects and tagnames.
				self.current_arg_object = self.arg_object_stack.pop()
				self.arg_tagname_stack.pop()
			else:
				# If list object is empty it must be a primitive, so replace
				# dict with pickup_content
				parent_tagname = self.arg_tagname_stack[-1:][0]
				parent_object = self.arg_object_stack[-1:][0]
				if len(parent_object[parent_tagname][-1:][0])==0:
					parent_object[parent_tagname][-1:] = [self.pickup_content]


	def characters(self,content):
		# Collect data continuosly, reset every time a new element is entered.
		# If it has been detected that the parser is in a CDATA Section all content
		# will be collected as is (raw) otherwise content is stripped.
		if self.state == self.ARGS_IN or self.cur_id_val:
			if self.in_cdata:
				# in CDATA Section
				self.pickup_content += content
			else:
				self.pickup_content += content.strip()


class SOAP11RequestHandler(BaseRequestHandler):
	
	def parse_request(self,soap_body,sinfo,encoding):
		parser = make_parser()
		ch = SOAP11ContentHandler(parser)
		parser.setContentHandler(ch)
		inpsrc = InputSource()
		inpsrc.setByteStream(BytesIO(soap_body))
		parser.parse(inpsrc)
		return ch.req_dict


class SOAP11ResponseHandler(BaseResponseHandler):
	
	_content_type = 'text/xml'
	_stringify_res_dict = True
	
	@staticmethod
	def value_to_soapxml(value,parent,doc,is_toplevel=False):
		if isinstance(value, dict):
			for attr_name,attr_val in value.items():
				xml_attr_name = attr_name.replace('_','-')
				attr_elem = doc.createElement(xml_attr_name)
				parent.appendChild(attr_elem)
				SOAP11ResponseHandler.value_to_soapxml(attr_val,attr_elem,doc)
		else:
			if is_toplevel:
				value_parent = doc.createElement('result')
				parent.appendChild(value_parent)
			else:
				value_parent = parent

			if isinstance(value, (list, tuple)):
				if not len(value):
					# Translate empty response arrays to SOAP Null (xsi:nil) value
					value_parent.setAttribute('xsi:nil','true')
				else:
					for item in value:
						item_element = doc.createElement('item')
						SOAP11ResponseHandler.value_to_soapxml(item,item_element,doc)
						value_parent.appendChild(item_element)
			else:
				if value==None:
					# Set xsi:nil to true if value is None
					value_parent.setAttribute('xsi:nil','true')
				else:
					value_parent.appendChild(doc.createTextNode(value))

	
	def build_response(self,res_dict,sinfo,encoding):
		import xml.dom.minidom as md
		doc = md.Document()
		envelope = doc.createElement('SOAP-ENV:Envelope')
		envelope.setAttribute('xmlns:SOAP-ENV','http://schemas.xmlsoap.org/soap/envelope/')
		envelope.setAttribute('xmlns:SOAP-ENC','http://schemas.xmlsoap.org/soap/encoding/')
		envelope.setAttribute('xmlns:xsd','http://www.w3.org/2001/XMLSchema')
		envelope.setAttribute('xmlns:ns','urn:%s' % res_dict['servicename'])
		doc.appendChild(envelope)
		body_elem = doc.createElement('SOAP-ENV:Body')
		body_elem.setAttribute('SOAP-ENV:encodingStyle','http://schemas.xmlsoap.org/soap/encoding/')
		envelope.appendChild(body_elem)
		method_elem = doc.createElement("ns:%sResponse" % res_dict['method'])
		if 'result' in res_dict['result']:
			SOAP11ResponseHandler.value_to_soapxml(res_dict['result'],method_elem,doc,is_toplevel=True)
		else:
			SOAP11ResponseHandler.value_to_soapxml({'result':res_dict['result']},method_elem,doc,is_toplevel=True)
		body_elem.appendChild(method_elem)
		return doc.toxml(encoding=encoding)

class SOAP11FaultHandler(BaseFaultHandler):
	
	_content_type = 'text/xml'
	_stringify_res_dict = True
	soapfault_template = """<?xml version='1.0' encoding='UTF-8'?>
<SOAP-ENV:Envelope
xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/1999/XMLSchema">
	<SOAP-ENV:Body>
		<SOAP-ENV:Fault>
			<faultcode xsi:type="xsd:string"></faultcode>
			<faultstring xsi:type="xsd:string"></faultstring>
			<detail></detail>
		</SOAP-ENV:Fault>
	</SOAP-ENV:Body>
</SOAP-ENV:Envelope>"""

	def build_fault_response(self,service_exc,sinfo,methodname,encoding,reflection):
		import xml.dom.minidom as md
		if service_exc.detail:
			detail = service_exc.detail
		else:
			detail = traceback.format_exc()
		d = md.parseString(self.soapfault_template)
		# Extract fault DOM elements
		faultcode_elem = d.getElementsByTagName('faultcode')[0]
		faultstring_elem = d.getElementsByTagName('faultstring')[0]
		detail_elem = d.getElementsByTagName('detail')[0]
		# Set the fault values
		faultcode_elem.appendChild(d.createTextNode(service_exc.faultcode))
		faultstring_elem.appendChild(d.createTextNode(service_exc.faultstring))
		detail_elem.appendChild(d.createTextNode(detail))
		# Return the SoapFault XML object
		return d.toxml(encoding=encoding)

@expose
class SOAP11Interface(BaseInterface):

	def __init__(self,sinfo,**kw):
		def_kw = {
			'service_descriptor': SOAP11ServiceDescriptor,
			'request_handler': SOAP11RequestHandler,
			'response_handler': SOAP11ResponseHandler,
			'fault_handler': SOAP11FaultHandler}
		def_kw.update(kw)
		BaseInterface.__init__(self,sinfo,**def_kw)

	@staticmethod
	def _interface_name():
		return 'soap11'

	@staticmethod
	def _accept_basetype(typ):
		return pytype_support.count(typ)>0

	@staticmethod
	def _accept_list():
		return True

	@staticmethod
	def _accept_dict():
		return False




# -*- coding: utf-8 -*-
import inspect,ast,re,pprint,sys,copy
from ladon.exceptions.ladonizer import *
import ladon.interfaces as interfaces
from ladon.types.ladontype import LadonType
from ladon.types import validate_type
from ladon.types.typemanager import TypeManager

# python2/3 support:
if sys.version_info[0]==2:
	from chardet_py2 import detect
	def get_function_name(f):
		return f.func_name

elif sys.version_info[0]>=3:
	from chardet_py3 import detect
	def get_function_name(f):
		return f.__name__



class LadonServiceCollection(object):
	
	"""
	LadonServiceCollection collects all services, methods and types for a given server-run
	In Ladon it is only used as a singleton global object, but this could change in time.
	The singleton behaviour is implemented in global_service_collection() which creates the
	global object the first time it is called and then returns the same object the rest of
	the time.
	
	add_service_method() is the most important method on this class as it is the central
	entry-point used by the ladonize decorator to register a service method.
	
	Ladon uses Abstract Syntax Trees (ast) to parse the source files associated with the
	service methods that are registered. We do that because the methods are unbound at
	call-time of the decorators by the Python interpreter. To actually detect the name of
	the class that will have the method as member we do a manual parse per source file.
	
	Example::
		LadonServiceCollection
		 + LadonServiceInfo (UserService)
		    + LadonMethodInfo (addUser)
		    + LadonMethodInfo (delUser)
		 + LadonServiceInfo (RoleService)
		    + LadonMethodInfo (addRole)
		    + LadonMethodInfo (delRole)
		    + LadonMethodInfo (addRoleMember)
		      .
		      .
	"""
	
	def __init__(self,collection_name='global'):
		"""
		initialize some members
		"""
		self.src_info = {}
		self.src_encoding = {}
		self.services = {}
		self.service_and_number = {}
		self.collection_name = collection_name
		self.service_counter = 1

	def source_info(self,fname):
		"""
		Parse a python source file at ast-level to internally acknowledge which
		class methods will be bound to which classes.
		"""
		# Has this source file already been parsed?
		if fname in self.src_info:
			# if yes return the previous parse-result
			return self.src_info[fname]
		
		# Create a source file parse-info-container and ast-parse the sourcefile
		self.src_info[fname] = {}
		src_fp = open(fname,'rb')
		src = src_fp.read()
		src_fp.close()
		src_encoding = detect(src)
		a = ast.parse(src)
		del src
		self.src_encoding[fname] = src_encoding['encoding']
		
		# Analyse the ast
		for obj in a.body:
			if type(obj)==ast.ClassDef:
				c = obj
				firstlineno = c.lineno
				lastlineno = c.lineno
				class_doc_lines = []
				first_class_obj = True
				for obj in c.body:
					# Detect documentation for class
					if first_class_obj and type(obj)==ast.Expr and type(obj.value)==ast.Str:
						for doc_line in inspect.cleandoc(obj.value.s.replace('\r\n','\n')).split('\n'):
							class_doc_lines += [doc_line]
					# Detect class methods
					if type(obj)==ast.FunctionDef:
						lastlineno = obj.lineno
					first_class_obj = False
				self.src_info[fname][c.name] = (firstlineno,lastlineno,class_doc_lines)
		
		# return the parse-info-container
		return self.src_info[fname]


	def add_service_method(self,f,*def_args,**def_kw):
		"""
		Register a class method. The method is expected to be registered a decorator
		and therefore unbound.
		If the class which the method is member of has not yet been registered a new
		LadonServiceInfo object will be created for it. The method itself will be
		associated with this object with the instantiation of a LadonMethodInfo
		object.
		"""
		# Extract the source filename and line number where the method is defined
		modulename = f.__module__
		src_fname = f.__code__.co_filename
		firstlineno = f.__code__.co_firstlineno
		# get an ast-analyzed object of the source file
		sinfo = self.source_info(src_fname)
		# Detect the name of the class that the unbound method will be member of
		
		for clsname,v in sinfo.items():
			if firstlineno>v[0] and firstlineno<=v[1]:
				# The method is somewhere within this class
				# Check if the class has been registered as a service yet
				if not (src_fname,clsname) in self.services:
					# If no then do it by creating a LadonServiceInfo object for it
					self.services[(src_fname,clsname)] = LadonServiceInfo(clsname,src_fname,modulename,firstlineno,v[2],self.service_counter,self.src_encoding[src_fname])
					self.service_and_number[self.service_counter] = (src_fname,clsname)
					self.service_counter += 1
				# Register the method as member of this class/service
				method = self.services[(src_fname,clsname)].add_method(f,*def_args,**def_kw)
				return method
		
		# Method could not be recognised as a class member
		return None
	
	def servicenames(self):
		"""
		Return a list of registered service names.
		"""
		names = []
		for k,v in self.services.items():
			names += [v.servicename]
		return names
		
	def service_by_source(self,sourcefile,servicename):
		"""
		Return a specific LadonServiceInfo object registered with a certain
		source file and a service name. Service names are actually class names
		and you cannot have 2 active classes with the same name in one module
		therefore this method will not end in an ambigious result.
		"""
		if (sourcefile,servicename) in self.services:
			return self.services[(sourcefile,servicename)]
		return None

	def service_by_number(self,servicenumber):
		"""
		Return a specific LadonServiceInfo object registered with a certain
		service number. Service numbers are unique, so this cannot end in an
		ambigious result.
		"""
		if servicenumber in self.service_and_number:
			return self.services[self.service_and_number[servicenumber]]
		return None

	def services_by_name(self,servicename):
		"""
		Return LadonServiceInfo objects by service name. This will almost
		always result in a single object, but it is possible to register
		methods from classes with identical names but implementation in
		different modules. This is perfectly legal and therefore it is
		possible to have more than one LadonServiceInfo object returned
		with this method.
		"""
		res = []
		for k,v in self.services.items():
			if k[1].lower() == servicename.lower():
				res += [self.services[k]]
		return res

	def services_by_function(self, fn):
		"""
		Return LadonServiceInfo objects by referenced service function (method).
		This will almost always result in a single object, but it is possible to register
		methods from classes with identical names but implementation in
		different modules. This is perfectly legal and therefore it is
		possible to have more than one LadonServiceInfo object returned
		with this method.
		
		@param fn: function (method) registered within the service
		@rtype: [LadonMethodInfo]
		"""
		source_info = self.source_info(fn.__code__.co_filename)
		source_line = fn.__code__.co_firstlineno
		
		for class_name,v in source_info.items() :
			if source_line > v[0] and source_line <= v[1] :
				return self.services_by_name(class_name)
		
		return []

	def serialize(self):
		"""
		Create an serialization object
		"""
		res = {'collection':self.collection_name}
		res['services'] = {}
		for k,sinfo in self.services.items():
			res['services'][k] = sinfo.serialize()
		return res

	def remove_service(self, sourcefile, servicename):
		"""Removes a service from collection
		
		@param servicename: name of service to remove
		@param sourcefile: filename with service source code
		@rtype: LadonServiceInfo  
		"""
		if self.has_service(sourcefile, servicename):
			k = (sourcefile, servicename)
			service = self.services[k]

			for num in self.service_and_number:
				if self.service_and_number[num] == k:
					break
	
			del self.services[k]
			del self.service_and_number[num]

			# rebuild service numbers
			new_service_and_number = {}
			counter = 1
			for old_num in sorted(self.service_and_number.keys()) :
				new_service_and_number[counter] = self.service_and_number[old_num]
				counter += 1

			self.service_and_number = new_service_and_number
			self.service_counter    = counter
	
			return service
		return False

	def services_remove_by_name(self, servicename):
		"""Removes a service or list of services with the given name from the
		service collection and returns the removed elements
		
		@param servicename: name of a service to remove 
		@rtype: [LadonServiceInfo] - list of removed service objects
		"""

		res = []
		services = copy.copy( self.services)
		for k in services:
			if k[1].lower() == servicename.lower():
				service = self.remove_service(k[0], k[1])
				if service:
					res += [service]

		return res

	def append_service(self, service):
		"""Appends a new service to the end of service collection
		
		@param service: LadonServiceInfo - service info object
		@rtype: bool
		"""
		k = (service.sourcefile, service.servicename)

		if not self.has_service(service.sourcefile, service.servicename):
			self.services[k] = service
			self.service_and_number[self.service_counter] = k
			self.service_counter += 1
	
			return True
		return False

	def has_service(self, sourcefile, servicename):
		"""Checks if current collection contains the service
		
		@param sourcefile: path to the source file with service definition
		@param servicename: name of service
		@rtype: bool
		"""
		return (sourcefile, servicename) in self.services

	def __str__(self):
		"""
		Make a pretty string representation
		"""
		return pprint.pformat({'collectionname':self.collection_name,'services':self.services})

class LadonServiceInfo(object):
	"""
	LadonServiceInfo stores information about a service that has been registered
	using LadonServiceCollection.add_service_method().
	It also aggragates all it's methods as LadonMethodInfo objects. A per service
	type manager (ladon.types.typemanager.TypeManager) that keeps track of the
	types that the service requires.
	"""
	def __init__(self,servicename,sourcefile,modulename,lineno,doc_lines,service_counter,src_encoding):
		self.servicename = servicename
		self.servicenumber = service_counter
		# store some info about the service
		self.src_encoding = src_encoding
		self.sourcefile = sourcefile
		self.modulename = modulename
		self.doc_lines = doc_lines
		self.lineno = lineno
		# The LadonMethodInfo object container
		self.methods = {}
		# Create the type manager for the service
		self.typemanager = TypeManager()
		
	def add_method(self,f,*def_args,**def_kw):
		"""
		Add a service method by creating a LadonMethodInfo object that analyses
		the function in regards to it's parameters and documentation.
		
		@param f The function object of the method to add
		@rtype LadonMethodInfo Return the method info of the method
		"""
		method = LadonMethodInfo(self,f,*def_args,**def_kw)
		# store the method info
		self.methods[get_function_name(f)] = method
		return method
	
	def method(self,methodname):
		"""
		Get the LadonMethodInfo object registered with *methodname*.
		
		@param methodname The name og the service method
		@rtype LadonMethodInfo The method info registered with methodname or None if non-existent
		"""
		if methodname in self.methods:
			return self.methods[methodname]
		return None
	
	def method_list(self):
		"""
		Get a list of the service methods
		
		@rtype list List of LadonMethodInfo objects
		"""
		method_names = list(self.methods.keys())
		method_names.sort()
		method_list = []
		for mn in method_names:
			method_list += [self.methods[mn]]
		return method_list
		
	def serialize(self):
		"""
		make a serialization object
		
		@rtype dict A serialization dictionary for the service
		"""
		res = {
			'servicename': self.servicename,
			'sourcefile': self.sourcefile,
			'self': self.lineno,
			'methods': {}}
		for k,minfo in self.methods.items():
			res['methods'][k] = minfo.serialize()
		return res
	
	def primitive_requirements(self):
		"""
		Get a list of the primitives required by the service.
		
		@rtype list A distinct list of primitives required by this service
		"""
		return self.typemanager.primitive_list

	def list_support_required(self):
		"""
		Query the service's typemanager to whether lists are required
		by the service. This information is used by the dispatcher to
		filter out service interfaces that don't support lists.
		
		@rtype bool Are lists required by the service
		"""
		return self.typemanager.has_lists

	def dict_support_required(self):
		"""
		Query the service's typemanager to whether dicts are required
		by the service. This information is used by the dispatcher to
		filter out service interfaces that don't support dictss.
		
		@rtype bool Are dicts required by the service
		"""
		return self.typemanager.has_dicts

	def append_method(self, methodinfo):
		"""Appends a method defined as LadonMethodInfo object to the current service
		
		@param methodinfo: LadonMethodInfo object
		@rtype: boolean 
		"""
		if not self.has_method(methodinfo.name()) :
			self.methods[methodinfo.name()] = methodinfo

			return True
		return False

	def remove_method(self, methodname):
		"""
		Removes the method with the given name from a service methods list
		
		@param methodname: name of the method to remove 
		"""
		if self.has_method(methodname) :
			method = self.methods[methodname]
			del self.methods[methodname]

			return method
		return False

	def has_method(self, methodname):
		"""
		Checks if the method with given name is registered in the service
		
		@param methodname: name of the method to check 
		"""
		return methodname in self.methods

	def __str__(self):
		"""
		Create a pretty string representation of the service info object
		
		@rtype String Pretty string
		"""
		return pprint.pformat({'servicename':self.servicename,'doc_lines':self.doc_lines,'sourcefile':self.sourcefile,'methods':self.methods})

# Regular expressions for documentation parsing 
rx_doc_params=re.compile('^@param\s+([_a-zA-Z0-9]+):\s*(.*)$')
rx_doc_rtype=re.compile('^@rtype:\s*(.*)$')
rx_doc_param_lines=re.compile('^\s+(\S.*)$')

class LadonMethodInfo(object):
	"""
	Structure for method information. Objects of this class holds method,
	arg and return type documentation and type-info. User-options defined
	via keyword args to the method like *encoding* and *allow_unsafe_conversion*
	are also stored here.
	"""
	def __init__(self,sinfo,f,*def_args,**def_kw):
		global rx_doc_params,rx_doc_rtype,rx_doc_param_lines
		# Inspect the methods's parameters
		argspecs = inspect.getargspec(f)
		# Get the method's doc-string
		doc = inspect.getdoc(f)
		if doc:
			if sys.version_info[0]==2:
				# Convert the documentation to unicode
				doc = unicode(doc,sinfo.src_encoding)
		# Associate the method with it's parent service-info object
		self.sinfo = sinfo
		# _args: Store for arguments (name, type, documentation and default value)
		self._args = {}
		# _has_keywords: Check if the method takes keyword arguments (**kw). The
		# dispatcher will use this information to decide whether to push ladon's
		# helper tools to the method. Helper tools are sent via keyword arguments
		# therefore keyword arguments are required in the method in order to recieve
		# the helper tools
		self._has_keywords = argspecs.keywords!=None
		# _arg_names,_defaults: Store method parameter names and defalut values here. 
		# These will be used later in the parsing process.
		self._arg_names = argspecs.args[1:]
		self._defaults = argspecs.defaults
		# Store the raw doc-string in _doc
		self._doc = f.__doc__
		# Extract the method name
		self._func_name = get_function_name(f)
		# Store the parameter types delivered by the user via the ladonize decorator
		self._arg_types = def_args
		self._real_method = f
		# _encoding: is kind of a contract between the method and Ladon's dispatcher.
		# It tells Ladon that it must deliver raw strings to the service method in that
		# encoding thus converting if nessecary. You can think of it as the method's
		# internal encoding. The method itself also commit's itself to the contract by
		# promising to deliver raw strings back to the dispatcher in the same encoding.
		# Raw strings are defined as: str in Python2 and bytes in Python3
		self._encoding = 'UTF-8'
		# _allow_unsafe_conversion: is a user-option that tells Ladon whether it may do
		# unsafe convertions or not. An example of an unsafe convertion is float to int.
		# unsafe convertions are disabled by default
		self._allow_unsafe_conversion = False
		# _rtype: The return type of the method.
		try:
			self._rtype = def_kw['rtype']
		except KeyError as ke:
			# return type must be specified.
			raise ReturnTypeUndefined(sinfo,self._func_name)
		# _rtype_doc: Initialize the return type doc-string container
		self._rtype_doc = []
		# overwrite the default values of user options if they are passed on via the
		# ladonize decorator.
		if 'encoding' in def_kw:
			self._encoding = def_kw['encoding']
		if 'allow_unsafe_conversion' in def_kw:
			self._allow_unsafe_conversion = def_kw['allow_unsafe_conversion']
		# Let the parent service's TypeManager object parse, analyze and store the return type
		self._multipart_response_required = sinfo.typemanager.analyze_param(self._rtype)
		self._multipart_request_required = False
		if self._defaults==None:
			self._defaults = tuple()
		# Identify which arguments are mandatory and which are optional
		self._mandatory_args = self._arg_names[:-len(self._defaults)]
		self._optional_args = self._arg_names[-len(self._defaults):]
		# Check that the number of type defines (via the ladonize decorator) matches then
		# number of method arguments
		if len(self._arg_names)!=len(def_args):
			raise ArgDefCountMismatch(sinfo,self._func_name,self._arg_types,self._arg_names)
		
		alen = len(self._arg_names)
		dlen = len(self._defaults)
		for argidx in range(len(self._arg_names)):
			# Let the parent service's TypeManager object parse, analyze and store the argument
			self._multipart_request_required |= sinfo.typemanager.analyze_param(def_args[argidx])

			has_default = False
			# find the index of this parameter's default value in _defaults. If index is below
			# 0 it means it has no default value, thus a mandatory parameter.
			defidx = dlen - (alen - argidx)
			if defidx>-1:
				# Optional parameter, check and store it's default value
				has_default = True
				def_val = self._defaults[defidx]
				def_type = type(self._defaults[defidx])
				# Check that the default value matches the user defined type
				if not validate_type(def_args[argidx],def_val):
					# type mismatch
					raise DefaultArgTypeMismatch(
						sinfo,
						self._func_name,
						self._arg_names[argidx],
						self._arg_types[argidx],
						def_type)

			# Store the info about this parameter
			self._args[self._arg_names[argidx]] = {
				'name': self._arg_names[argidx],
				'type': def_args[argidx],
				'optional': has_default,
				'doc': []}
			if has_default:
				self._args[self._arg_names[argidx]]['default'] = def_val
		
		# Parse the methods documentation
		self._method_doc = []
		if doc:
			# Make sure all newlines in doc are Unix style, then split it into lines
			doc = doc.replace('\r\n','\n')
			doclines = doc.split('\n')
			
			cur_argname = None
			for dline in doclines:
				# Iterate through doc lines and do regular expression matches to
				# find method, argument and return documentation
				
				# Does the current doc line match the beginning of a parameters documentation
				param_match = rx_doc_params.match(dline)
				if param_match:
					# Set the parameter as current
					cur_argname = param_match.groups()[0]
					# if not an actual parameter cancel param documentation
					if not cur_argname in self._args:
						cur_argname = None
						continue
					# Add the first doc line to the parameter doc lines
					docline1 = param_match.groups()[1].strip()
					if docline1:
						self._args[cur_argname]['doc'] += [docline1]
					continue
				
				# Does the current doc line match the beginning of the return documentation
				rtype_match = rx_doc_rtype.match(dline)
				if rtype_match:
					# Add the first doc line to the return doc lines
					cur_argname = '_rtype'
					docline1 = rtype_match.groups()[0].strip()
					if docline1:
						self._rtype_doc += [docline1]
					continue
				
				# does the doc line look like a param or return line
				param_line_match = rx_doc_param_lines.match(dline)
				# Add it to the current param doc that is being processed
				if param_line_match and cur_argname:
					if cur_argname=='_rtype':
						self._rtype_doc += [ dline.strip() ]
					else:
						self._args[cur_argname]['doc'] += [param_line_match.groups()[0].strip()]
					continue
				
				# If this doc line cannot be mapped to an argument or the return documentation, then
				# add it to the methods documentation
				if not len(dline.strip()) and cur_argname:
					if cur_argname=='_rtype':
						self._rtype_doc += [ dline.strip() ]
					else:
						self._args[cur_argname]['doc'] += [dline.strip()]
					continue
				
				# If no match found, reset the current param if one is in focus
				cur_argname = None
				self._method_doc += [ dline ]
				
		self.__doc__ = '\n'.join(self._method_doc)
		
	def name(self):
		"""
		Fetch the name of the method being mapped by this LadonMethodInfo.
		
		@param rtype String The name of the method
		"""
		return self._func_name

	def args(self):
		"""
		Fetch information about this methods parameters.
		
		@rtype list A list of argument description dicts
		"""
		ret = []
		for argname in self._arg_names:
			ret += [self._args[argname]]
		return ret

	def serialize(self):
		"""
		Serialize this method
		"""
		return {'methodname':self.name(),'doc':self.__doc__,'args':self.args(),'rtype':self._rtype, 'rtype_doc':self._rtype_doc}

	def get_real_method(self):
		"""
		Returns a reference to the real service method was decorated by ladonizer
		
		@rtype: function which was decorated
		"""
		return self._real_method

	def add_arg(self, arg_name, arg_type, arg_doc, is_optional = False, insert_before=None, insert_after=None):
		"""Adds a new argument to the method. It is possible to add an argument at specified position
		by setting up the insert_before or insert_after argument name. By the way there are dedicated
		methods existing as aliases to an appropriate operation - see LadonMethodInfo:arg_insert_before()
		and LadonMethodInfo:arg_insert_after()
		Please, note, that only one of insert_before or insert_after parameter will be taken into account
		and insert_before is a high priority. It means that bypassing both parameters at a time
		will cause selection of insert_before behavior, insert_after will be ommited
		By default the argument will be added to the end of service method's arguments list
		Returns True if operation successfull, False otherwise
	
		@param arg_name: a name of an argument to add to a service method
		@param arg_type: an argument type defined for a service method
		@param arg_doc: a docuentation string(s) to add to a service method
		@param is_optional: specifies if an argument should be treated as optional or not at a service method
		@param insert_before: specifies a name of service method's argument before which the new argument should be added
		@param insert_after: specifies a name of service method's argument after which the new argument should be added
		@rtype: bool
		"""
		if not self.has_arg(arg_name):
			if type(arg_doc) is str:
				arg_doc = [arg_doc]
	
			self._args[arg_name] = {
				'doc'      : arg_doc,
				'name'     : arg_name,
				'optional' : is_optional,
				'type'     : arg_type
			}
	
			if insert_before is not None :
				argidx = self._arg_names.index(insert_before)
	
			elif insert_after is not None :
				argidx = self._arg_names.index(insert_after) + 1
	
			else :
				argidx = len(self._arg_names)
	
			self._arg_names[argidx:1] = [arg_name]
	
			atlist = list(self._arg_types)
			atlist[argidx:1] = [arg_type]
			self._arg_types = tuple(atlist)

			return True
		return False

	def arg_insert_before(self, ref_arg_name, arg_name, arg_type, arg_doc, is_optional = False):
		"""
		An alias for LadonMethodInfo:add_arg() with the insert_before parameter specified as ref_arg_name
		Returns True if operation successfull, False otherwise
		
		@param ref_arg_name: name of service method's argument before which the new argument should be added
		@param arg_name: a name of an argument to add to a service method
		@param arg_type: an argument type defined for a service method
		@param arg_doc: a docuentation string(s) to add to a service method
		@param is_optional: specifies if an argument should be treated as optional or not at a service method
		@rtype: bool
		"""
		return self.add_arg(arg_name, arg_type, arg_doc, is_optional, insert_before=ref_arg_name)

	def arg_insert_after(self, ref_arg_name, arg_name, arg_type, arg_doc, is_optional = False):
		"""
		An alias for LadonMethodInfo:add_arg() with the insert_after parameter specified as ref_arg_name
		Returns True if operation successfull, False otherwise
		
		@param ref_arg_name: name of service method's argument after which the new argument should be added
		@param arg_name: a name of an argument to add to a service method
		@param arg_type: an argument type defined for a service method
		@param arg_doc: a docuentation string(s) to add to a service method
		@param is_optional: specifies if an argument should be treated as optional or not at a service method
		@rtype: bool
		"""
		return self.add_arg(arg_name, arg_type, arg_doc, is_optional, insert_after=ref_arg_name)

	def remove_arg(self, arg_name):
		"""
		Removes an argument with the name specified from a registered arguments for current service method
		Returns removed argument representation or False if argument is not present in the method's arument list
		
		@param arg_name: name of a service method's argument to remove
		@rtype: dict { "doc" : [str], "name" : str, "optional" : bool, "type" : LadonType } or bool False
		"""
		if self.has_arg(arg_name):
			arg = self._args[arg_name]
			argidx = self._arg_names.index(arg_name)
	
			del self._arg_names[argidx]
			del self._args[arg_name]

			atlist = list(self._arg_types)
			del atlist[argidx]
			self._arg_types = tuple(atlist)

			return arg
		return False

	def has_arg(self, arg_name):
		"""
		Checks if the argument with the name specified exists in the list of service method arguments
		
		@param arg_name: name of an argument to check
		@rtype: bool 
		"""
		return arg_name in self._args

	def __str__(self):
		return pprint.pformat(self.serialize())

global_lsc = None

# Singleton
def global_service_collection():
	"""
	Fetch the singleton instance of LadonServiceCollection refered to as the
	global service collection.
	
	@rtype LadonServiceCollection The global service collection
	"""
	global global_lsc
	# If this is the first call then the object is not yet created
	if not global_lsc:
		# Create the global object
		global_lsc = LadonServiceCollection()
	return global_lsc



from ladon.exceptions.base import LadonException
import sys,os,traceback

class ServiceFault(LadonException):
	def __init__(self,faultcode,faultstring,detail,stack_depth):
		super(ServiceFault,self).__init__()
		self.faultstring = faultstring
		self.detail = detail
		self.faultcode = faultcode
		caller = sys._getframe(stack_depth)
		self.func = caller.f_code.co_name
		self.mod = os.path.splitext(os.path.split(caller.f_code.co_filename)[1])[0]
		self.lineno = caller.f_lineno
	
	def __str__(self):
		return "%s(%d): %s" % (self.mod,self.lineno,self.faultstring)


class ClientFault(ServiceFault):
	def __init__(self,faultstring,detail=None,stack_depth=2):
		super(ClientFault,self).__init__('client',faultstring,detail,stack_depth)

class ServerFault(ServiceFault):
	def __init__(self,faultstring,detail=None,stack_depth=2):
		super(ServerFault,self).__init__('server',faultstring,detail,stack_depth)



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Communicator client code.
# 
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998
# the Initial Developer. All Rights Reserved.
# 
# Contributor(s):
#   Mark Pilgrim - port to Python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from . import constants
from .euctwfreq import EUCTWCharToFreqOrder, EUCTW_TABLE_SIZE, EUCTW_TYPICAL_DISTRIBUTION_RATIO
from .euckrfreq import EUCKRCharToFreqOrder, EUCKR_TABLE_SIZE, EUCKR_TYPICAL_DISTRIBUTION_RATIO
from .gb2312freq import GB2312CharToFreqOrder, GB2312_TABLE_SIZE, GB2312_TYPICAL_DISTRIBUTION_RATIO
from .big5freq import Big5CharToFreqOrder, BIG5_TABLE_SIZE, BIG5_TYPICAL_DISTRIBUTION_RATIO
from .jisfreq import JISCharToFreqOrder, JIS_TABLE_SIZE, JIS_TYPICAL_DISTRIBUTION_RATIO

ENOUGH_DATA_THRESHOLD = 1024
SURE_YES = 0.99
SURE_NO = 0.01

class CharDistributionAnalysis:
    def __init__(self):
        self._mCharToFreqOrder = None # Mapping table to get frequency order from char order (get from GetOrder())
        self._mTableSize = None # Size of above table
        self._mTypicalDistributionRatio = None # This is a constant value which varies from language to language, used in calculating confidence.  See http://www.mozilla.org/projects/intl/UniversalCharsetDetection.html for further detail.
        self.reset()
        
    def reset(self):
        """reset analyser, clear any state"""
        self._mDone = False # If this flag is set to True, detection is done and conclusion has been made
        self._mTotalChars = 0 # Total characters encountered
        self._mFreqChars = 0 # The number of characters whose frequency order is less than 512

    def feed(self, aBuf, aCharLen):
        """feed a character with known length"""
        if aCharLen == 2:
            # we only care about 2-bytes character in our distribution analysis
            order = self.get_order(aBuf)
        else:
            order = -1
        if order >= 0:
            self._mTotalChars += 1
            # order is valid
            if order < self._mTableSize:
                if 512 > self._mCharToFreqOrder[order]:
                    self._mFreqChars += 1

    def get_confidence(self):
        """return confidence based on existing data"""
        # if we didn't receive any character in our consideration range, return negative answer
        if self._mTotalChars <= 0:
            return SURE_NO

        if self._mTotalChars != self._mFreqChars:
            r = self._mFreqChars / ((self._mTotalChars - self._mFreqChars) * self._mTypicalDistributionRatio)
            if r < SURE_YES:
                return r

        # normalize confidence (we don't want to be 100% sure)
        return SURE_YES

    def got_enough_data(self):
        # It is not necessary to receive all data to draw conclusion. For charset detection,
        # certain amount of data is enough
        return self._mTotalChars > ENOUGH_DATA_THRESHOLD

    def get_order(self, aBuf):
        # We do not handle characters based on the original encoding string, but 
        # convert this encoding string to a number, here called order.
        # This allows multiple encodings of a language to share one frequency table.
        return -1
    
class EUCTWDistributionAnalysis(CharDistributionAnalysis):
    def __init__(self):
        CharDistributionAnalysis.__init__(self)
        self._mCharToFreqOrder = EUCTWCharToFreqOrder
        self._mTableSize = EUCTW_TABLE_SIZE
        self._mTypicalDistributionRatio = EUCTW_TYPICAL_DISTRIBUTION_RATIO

    def get_order(self, aBuf):
        # for euc-TW encoding, we are interested 
        #   first  byte range: 0xc4 -- 0xfe
        #   second byte range: 0xa1 -- 0xfe
        # no validation needed here. State machine has done that
        if aBuf[0] >= 0xC4:
            return 94 * (aBuf[0] - 0xC4) + aBuf[1] - 0xA1
        else:
            return -1

class EUCKRDistributionAnalysis(CharDistributionAnalysis):
    def __init__(self):
        CharDistributionAnalysis.__init__(self)
        self._mCharToFreqOrder = EUCKRCharToFreqOrder
        self._mTableSize = EUCKR_TABLE_SIZE
        self._mTypicalDistributionRatio = EUCKR_TYPICAL_DISTRIBUTION_RATIO

    def get_order(self, aBuf):
        # for euc-KR encoding, we are interested 
        #   first  byte range: 0xb0 -- 0xfe
        #   second byte range: 0xa1 -- 0xfe
        # no validation needed here. State machine has done that
        if aBuf[0] >= 0xB0:
            return 94 * (aBuf[0] - 0xB0) + aBuf[1] - 0xA1
        else:
            return -1;

class GB2312DistributionAnalysis(CharDistributionAnalysis):
    def __init__(self):
        CharDistributionAnalysis.__init__(self)
        self._mCharToFreqOrder = GB2312CharToFreqOrder
        self._mTableSize = GB2312_TABLE_SIZE
        self._mTypicalDistributionRatio = GB2312_TYPICAL_DISTRIBUTION_RATIO

    def get_order(self, aBuf):
        # for GB2312 encoding, we are interested 
        #  first  byte range: 0xb0 -- 0xfe
        #  second byte range: 0xa1 -- 0xfe
        # no validation needed here. State machine has done that
        if (aBuf[0] >= 0xB0) and (aBuf[1] >= 0xA1):
            return 94 * (aBuf[0] - 0xB0) + aBuf[1] - 0xA1
        else:
            return -1;

class Big5DistributionAnalysis(CharDistributionAnalysis):
    def __init__(self):
        CharDistributionAnalysis.__init__(self)
        self._mCharToFreqOrder = Big5CharToFreqOrder
        self._mTableSize = BIG5_TABLE_SIZE
        self._mTypicalDistributionRatio = BIG5_TYPICAL_DISTRIBUTION_RATIO

    def get_order(self, aBuf):
        # for big5 encoding, we are interested 
        #   first  byte range: 0xa4 -- 0xfe
        #   second byte range: 0x40 -- 0x7e , 0xa1 -- 0xfe
        # no validation needed here. State machine has done that
        if aBuf[0] >= 0xA4:
            if aBuf[1] >= 0xA1:
                return 157 * (aBuf[0] - 0xA4) + aBuf[1] - 0xA1 + 63
            else:
                return 157 * (aBuf[0] - 0xA4) + aBuf[1] - 0x40
        else:
            return -1

class SJISDistributionAnalysis(CharDistributionAnalysis):
    def __init__(self):
        CharDistributionAnalysis.__init__(self)
        self._mCharToFreqOrder = JISCharToFreqOrder
        self._mTableSize = JIS_TABLE_SIZE
        self._mTypicalDistributionRatio = JIS_TYPICAL_DISTRIBUTION_RATIO

    def get_order(self, aBuf):
        # for sjis encoding, we are interested 
        #   first  byte range: 0x81 -- 0x9f , 0xe0 -- 0xfe
        #   second byte range: 0x40 -- 0x7e,  0x81 -- oxfe
        # no validation needed here. State machine has done that
        if (aBuf[0] >= 0x81) and (aBuf[0] <= 0x9F):
            order = 188 * (aBuf[0] - 0x81)
        elif (aBuf[0] >= 0xE0) and (aBuf[0] <= 0xEF):
            order = 188 * (aBuf[0] - 0xE0 + 31)
        else:
            return -1;
        order = order + aBuf[1] - 0x40
        if aBuf[1] > 0x7F:
            order =- 1
        return order

class EUCJPDistributionAnalysis(CharDistributionAnalysis):
    def __init__(self):
        CharDistributionAnalysis.__init__(self)
        self._mCharToFreqOrder = JISCharToFreqOrder
        self._mTableSize = JIS_TABLE_SIZE
        self._mTypicalDistributionRatio = JIS_TYPICAL_DISTRIBUTION_RATIO

    def get_order(self, aBuf):
        # for euc-JP encoding, we are interested 
        #   first  byte range: 0xa0 -- 0xfe
        #   second byte range: 0xa1 -- 0xfe
        # no validation needed here. State machine has done that
        if aBuf[0] >= 0xA0:
            return 94 * (aBuf[0] - 0xA1) + aBuf[1] - 0xa1
        else:
            return -1
